@@ -1,9 +1,11 @@
 /*
-  Copyright(C) 2016 Brazil
+  Copyright (C) 2016  Brazil
+  Copyright (C) 2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License version 2.1 as published by the Free Software Foundation.
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,8 +21,14 @@
 
 #include "grn.h"
 
-grn_bool grn_request_timer_init(void);
-void grn_request_timer_fin(void);
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+bool
+grn_request_timer_init(void);
+void
+grn_request_timer_fin(void);
 
 #ifdef __cplusplus
 }
